import requests
from bs4 import BeautifulSoup
from phdictionary.purifier import Purifier


class Requester:

    def get_definition_1(self, query='hello', number_of_examples=0, grammar_mark=False):
        r = requests.get('https://www.collinsdictionary.com/dictionary/english/' + query.replace(' ', '-'),
                         headers={'User-agent': 'Mozilla/5.0'})
        p = Purifier()
        return p.purify_definition(r.text, number_of_examples)

    def get_french_english_q(self, query='bonsoir', number_of_examples=0, grammar_mark=False):
        r = requests.get('https://www.collinsdictionary.com/dictionary/french-english/' + str(query).replace(' ', '-'),
                         headers={'User-agent': 'Mozilla/5.0'})
        p = Purifier()
        return p.purify_fr_eng(r.text, number_of_examples)

    def get_synonym(self, query='hello', number_of_synonyms=1):
        r = requests.get('https://www.thesaurus.com/browse/' + query.replace(' ', '-'),
                         headers={'User-agent': 'Mozilla/5.0'})
        p = Purifier()
        return p.purify_syn_eng(r.text, number_of_synonyms)

    def get_eng_fr(self, query='bonsoir', number_of_examples=0, grammar_mark=False):
        r = requests.get('https://www.collinsdictionary.com/dictionary/english-french/' + str(query).replace(' ', '-'),
                         headers={'User-agent': 'Mozilla/5.0'})
        p = Purifier()
        return p.purify_fr_eng(r.text, number_of_examples)

    def get_glosbe(self, query='bonsoir', number_of_examples=0, language='fr-ru'):
        if language == 'fr-ru':

            r = requests.get(f'https://glosbe.com/fr/ru/{query}',
                             headers={'User-agent': 'Mozilla/5.0'})
            p = Purifier()
            return p.purify_glosbe(r.text, number_of_examples)
        elif language == 'ru-fr':
            r = requests.get(f'https://glosbe.com/ru/fr/{query}',
                             headers={'User-agent': 'Mozilla/5.0'})
            p = Purifier()
            return p.purify_glosbe(r.text, number_of_examples)
        elif language == 'en-ru':
            r = requests.get(f'https://glosbe.com/en/ru/{query}',
                             headers={'User-agent': 'Mozilla/5.0'})
            p = Purifier()
            return p.purify_glosbe(r.text, number_of_examples)
        elif language == 'ru-en':
            r = requests.get(f'https://glosbe.com/ru/en{query}',
                             headers={'User-agent': 'Mozilla/5.0'})
            p = Purifier()
            return p.purify_glosbe(r.text, number_of_examples)



from phdictionary.dictionary import get_definition, \
    get_synonym, \
    get_french_english, \
    get_english_french, \
    get_synonyms_from_file, \
    get_french_english_from_file, \
    get_definition_from_file, \
    get_english_french_from_file, \
    get_french_russian, \
    get_russian_french, \
    get_russian_english, \
    get_english_russian



from phdictionary.requestor import Requester
from phdictionary.file_manager import FileManager


# Definition block
# {
def get_definition(word, number_of_examples=0, grammar_mark=False):
    r = Requester()
    if grammar_mark == False and number_of_examples == 0:
        return r.get_definition_1(word, number_of_examples, grammar_mark)[1:2]
    elif grammar_mark == True and number_of_examples == 0:
        return r.get_definition_1(word, number_of_examples, grammar_mark)[:2]
    elif number_of_examples != 0 and grammar_mark == False:
        return r.get_definition_1(word, number_of_examples, grammar_mark)[1:]
    return r.get_definition_1(word, number_of_examples, grammar_mark)


# }

# Translation block
# {

def get_french_english(word, number_of_examples=0, grammar_mark=False):
    try:
        r = Requester()
        if grammar_mark == False and number_of_examples == 0:
            return r.get_french_english_q(word, number_of_examples, grammar_mark)[1:2]
        elif grammar_mark == True and number_of_examples == 0:
            return r.get_french_english_q(word, number_of_examples, grammar_mark)[:2]
        elif number_of_examples != 0 and grammar_mark == False:
            return r.get_french_english_q(word, number_of_examples, grammar_mark)[1:]
        return r.get_french_english_q(word, number_of_examples, grammar_mark)
    except:
        print('Try again, something went wrong. It might be spelling')


def get_english_french(word, number_of_examples=0, grammar_mark=False):
    try:
        r = Requester()
        if grammar_mark == False and number_of_examples == 0:
            return r.get_eng_fr(word, number_of_examples, grammar_mark)[1:2]
        elif grammar_mark == True and number_of_examples == 0:
            return r.get_eng_fr(word, number_of_examples, grammar_mark)[:2]
        elif number_of_examples != 0 and grammar_mark == False:
            return r.get_eng_fr(word, number_of_examples, grammar_mark)[1:]
        return r.get_eng_fr(word, number_of_examples, grammar_mark)
    except:
        print('Try again, something went wrong. It might be spelling')


def get_french_russian(word, number_of_examples=0, ):
    r = Requester()
    return r.get_glosbe(word, number_of_examples, 'fr-ru')


def get_russian_french(word, number_of_examples=0):
    r = Requester()
    return r.get_glosbe(word, number_of_examples, 'ru-fr')


def get_russian_english(word, number_of_examples=0):
    r = Requester()
    return r.get_glosbe(word, number_of_examples, 'ru-en')


def get_english_russian(word, number_of_examples=0):
    r = Requester()
    return r.get_glosbe(word, number_of_examples, 'en-ru')


# }
# Synonym block
# {
def get_synonym(word, number_of_syn=1):
    try:
        r = Requester()
        return r.get_synonym(word, number_of_syn)
    except:
        print('Try again, something went wrong. It might be spelling')


# }
# Files block
# {
def get_french_english_from_file(file, number_of_examples=0):
    f = FileManager()
    return f.txt_french_english(file, number_of_examples)


def get_synonyms_from_file(file, number_of_syn=1):
    f = FileManager()
    return f.syn_txt(file, number_of_syn)


def get_definition_from_file(file, number_of_examples=0):
    f = FileManager()
    return f.definition(file, number_of_examples)


def get_english_french_from_file(file, number_of_examples=0):
    f = FileManager()
    return f.txt_english_french(file, number_of_examples)
# }



from phdictionary.requestor import Requester
from docx import Document
from random import randint


class FileManager:
    def txt_french_english(self, f, number_of_examples=0):
        r = Requester()
        f = f.read().splitlines()
        if number_of_examples == 0:
            document = Document()
            document.add_heading('French-English', 0)
            for word in f:
                if word != '':
                    re = r.get_french_english_q(word, 0)[:2]
                    if re[0] == 'feminine noun':
                        word += ' (f)'
                    elif re[0] == 'masculine noun':
                        word += ' (m) '
                    document.add_paragraph(word + '  -  ' + re[1], style='List Bullet')
            document.save(f'french-english{randint(0, 1000)}.docx')
        else:
            document = Document()
            document.add_heading('French-English', 0)
            for word in f:
                if word != '':
                    re = r.get_french_english_q(word, number_of_examples)
                    if re[0] == 'feminine noun':
                        word += ' (f)'
                    elif re[0] == 'masculine noun':
                        word += ' (m) '
                    p = document.add_paragraph()
                    p.add_run(word + '  -  ' + re[1]).bold = True
                    for ex in re[2]:
                        document.add_paragraph(ex, style='List Bullet')

            document.save(f'french-english-examples{randint(0, 1000)}.docx')

    def syn_txt(self, f, number_of_syn):
        r = Requester()
        f = f.read().splitlines()
        document = Document()
        document.add_heading('Synonyms', 0)
        for word in f:
            if word != '':
                re = r.get_synonym(str(word), number_of_syn)
                p = document.add_paragraph(style='List Bullet')
                to_add = ''

                for i in range(len(re)):
                    re[i].replace('%20', ' ')
                    if i == len(re) - 1:
                        to_add += re[i]
                    else:
                        to_add += re[i] + ', '
                p.add_run(word + '  -  ' + to_add).bold = True
        document.save(f'synonyms{randint(0, 1000)}.docx')

    def definition(self, f, number_of_examples):
        f = f.read().splitlines()
        r = Requester()
        if number_of_examples == 0:
            document = Document()
            document.add_heading('Definition', 0)
            for word in f:
                if word != '':
                    re = r.get_definition_1(word, 0)[:2]
                    if '\n' not in word:
                        p = document.add_paragraph(style='List Bullet')
                        p.add_run(word[0].upper() + word[1:]).bold = True
                        p.add_run('  -  ' + re[1])
                    else:
                        word = word[0].upper() + word[1:len(word) - 1]
                        p = document.add_paragraph(style='List Bullet')
                        p.add_run(word).bold = True
                        p.add_run('  -  ' + re[1])
            document.save(f'definition{randint(0, 1000)}.docx')
        else:
            document = Document()
            document.add_heading('Definition', 0)
            for word in f:
                if word != '':
                    re = r.get_definition_1(word, number_of_examples)
                    if '\n' not in word:
                        p = document.add_paragraph()
                        p.add_run(word[0].upper() + word[1:]).bold = True
                        p.add_run('  -  ' + re[1])
                    else:
                        word = word[0].upper() + word[1:len(word) - 1]
                        p = document.add_paragraph()
                        p.add_run(word).bold = True
                        p.add_run('  -  ' + re[1])
                    for ex in re[2]:
                        document.add_paragraph(ex, style='List Bullet')
            document.save(f'definition_examples{randint(0, 1000)}.docx')

    def txt_english_french(self, f, number_of_examples=0):
        r = Requester()
        f = f.read().splitlines()
        if number_of_examples == 0:
            document = Document()
            document.add_heading('English-French', 0)
            for word in f:
                if word != '':
                    re = r.get_eng_fr(word, 0)[:2]
                    if re[0] == 'feminine noun':
                        word += ' (f)'
                    elif re[0] == 'masculine noun':
                        word += ' (m) '
                    document.add_paragraph(word + '  -  ' + re[1], style='List Bullet')
            document.save(f'english-french{randint(0, 1000)}.docx')
        else:
            document = Document()
            document.add_heading('English-French', 0)
            for word in f:
                if word != '':
                    re = r.get_eng_fr(word, number_of_examples)
                    if re[0] == 'feminine noun':
                        word += ' (f)'
                    elif re[0] == 'masculine noun':
                        word += ' (m) '
                    p = document.add_paragraph()
                    p.add_run(word + '  -  ' + re[1]).bold = True
                    for ex in re[2]:
                        document.add_paragraph(ex, style='List Bullet')

            document.save(f'english-french-examples{randint(0, 1000)}.docx')



from bs4 import BeautifulSoup


class Purifier:
    def purify(self, a):
        try:
            purified = ''
            add = False
            for i in range(len(a)):
                if a[i] == '>':
                    add = True
                elif a[i] == '<':
                    add = False
                elif a[i] == '\n':
                    pass
                elif add:
                    purified += a[i]
            if purified[0] == ' ':
                purified = purified[1:]
            return purified
        except:
            return ''

    def purify_syn_english(self, a):
        try:
            purified = ''
            add = False
            for i in range(len(a)):
                if a[i] == '/':
                    add = True
                elif a[i] == '"':
                    add = False
                elif a[i] == '\n':
                    pass
                elif add:
                    purified += a[i]
            if purified[0] == ' ':
                purified = purified[1:]
            purified = purified[6:len(purified) - 2]
            return purified
        except:
            print('Only this amount of appropriate synonyms has found')
            return ''

    def purify_fr_eng(self, text, number_of_examples):
        p = Purifier()
        soap = BeautifulSoup(text, 'html.parser')
        grammar = str(soap.find(class_='pos'))
        translation = str(soap.find(class_='cit type-translation quote'))
        examples = soap.find_all(class_='quote')
        p_examples = list()
        for i in range(len(examples) - 1, len(examples) - number_of_examples - 1, -1):
            p_examples.append(p.purify(str(examples[i])))
        return p.purify(grammar), p.purify(translation), p_examples

    def purify_definition(self, text, number_of_examples):
        p = Purifier()
        soap = BeautifulSoup(text, 'html.parser')
        grammar = str(soap.find(class_='gramGrp pos'))
        definition = str(soap.find(class_='def'))
        examples = soap.find_all(class_='quote')
        p_examples = list()
        for i in range(len(examples) - 1, len(examples) - number_of_examples - 1, -1):
            p_examples.append(p.purify(str(examples[i])))
        return p.purify(grammar), p.purify(definition), p_examples

    def purify_syn_eng(self, text, number_of_synonyms):
        p = Purifier()
        soap = BeautifulSoup(text, 'html.parser')
        syn = soap.find_all(class_='css-1kg1yv8 eh475bn0')
        syn_res = list()
        if len(syn) < number_of_synonyms:
            additional = syn + soap.find_all(class_='css-1gyuw4i eh475bn0')
            for i in range(number_of_synonyms):
                try:
                    syn_res.append(p.purify_syn_english(str(additional[i])))
                except:
                    syn_res.append('')
                    print('Currently it is not possible to find so many synonyms')
            return syn_res
        else:
            for i in range(number_of_synonyms):
                syn_res.append(p.purify_syn_english(str(syn[i])))
            return syn_res

    def purify_glosbe(self, text, number_of_examples):
        p = Purifier()
        soap = BeautifulSoup(text, 'html.parser')
        s = soap.find_all(class_='translation')
        translation = []
        examples = []

        for el in s:
            translation.append(p.purify(str(el)))

        s = soap.find_all(class_='translation__example')

        for j in range(number_of_examples):
            res = p.purify(str(s[j]))
            r = ''
            split = True
            for i in range(len(res)):
                if ord(res[i]) > 1000 and split:
                    split = False
                    r += ' || '
                    r += res[i]
                else:
                    r += res[i]
            examples.append(r)

        return translation, examples
        # s = soap.find_all(class_='phrase__summary__field')
        # for el in s:
        #     purify_french_russian(str(el))
